/**
* Majority_black_2D.java
* Created on 2005 by Manuel Guillermo Forero-Vargas
* e-mail: mgforero@yahoo.es
* 
* Function: Majority black filter algorithm. A white pixel is eliminated if is
 * surrounded by only two white pixels or less. 
*
* This plugin do not create a new image
*
* Copyright (c) 2005 by Manuel Guillermo Forero-Vargas
*
* This plugin is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this plugin; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;

public class Majority_black_2D implements PlugInFilter
 {
    protected   ImagePlus   imp;
    int height,width;
    
    public int setup(String arg, ImagePlus imp)
    {
        if (arg.equals("about"))
        {
            showAbout();
            return DONE;
        }
        if (imp==null)
        {
            IJ.noImage();
            return DONE;
        }
        ImageStatistics stats=imp.getStatistics();
        if (stats.histogram[0]+stats.histogram[255]!=stats.pixelCount)
        {
            IJ.error("8-bit binary image (0 and 255) required.");
            return DONE;
        }
        this.imp=imp;
        height=imp.getHeight();
        width=imp.getWidth();
        //----------------------------------------------------------------------
        //Ask if you want to apply the method to the stack or just to the image
        return IJ.setupDialog(imp,DOES_8G+DOES_8C);
    }

    public void run(ImageProcessor ip)
    {
        int     a,x,y;
	int[][]	pixels=new int [width+2][height+2];

        //----------------------------------------------------------------------
        //A copy of the original image is done
        //----------------------------------------------------------------------
        //Creates a copy of the original image
        byte[] pPixels=(byte[])ip.getPixels();
        for(a=y=0;y<height;y++)
            for(x=0;x<width;x++)
                pixels[x+1][y+1]=pPixels[a++]&0xff;//.getPixel(x,y);//Original
        //----------------------------------------------------------------------
        //The majority black filter is applied
	for (y=1;y<=height;y++)
            for(x=1;x<=width;x++)
                if(pixels[x][y]>0&(pixels[x-1][y-1]+pixels[x][y-1]+pixels[x+1][y-1]+pixels[x-1][y]+pixels[x+1][y]+pixels[x-1][y+1]+pixels[x][y+1]+pixels[x+1][y+1])<=510)
                    ip.putPixel(x-1,y-1,0);
        //----------------------------------------------------------------------
        imp.show();
        imp.updateAndDraw();
    }

    void showAbout()
    {
        IJ.showMessage("About Majority_black_2D...",
        "This plug-in filter eliminates white pixels surrounded by more than a quantity of black pixels.");
    }
}
